package cn.iocoder.yudao.module.smartfactory.service.staffsalary;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.smartfactory.controller.admin.staffsalary.vo.StaffSalaryPageReqVO;
import cn.iocoder.yudao.module.smartfactory.controller.admin.staffsalary.vo.StaffSalaryRespVO;
import cn.iocoder.yudao.module.smartfactory.controller.admin.staffsalary.vo.StaffSalarySaveReqVO;
import cn.iocoder.yudao.module.smartfactory.dal.dataobject.staffsalary.StaffSalaryDO;
import cn.iocoder.yudao.module.smartfactory.dal.mysql.staff.StaffMapper;
import cn.iocoder.yudao.module.smartfactory.dal.mysql.staffsalary.StaffSalaryMapper;
import cn.iocoder.yudao.module.system.api.loan.LoanApi;
import cn.iocoder.yudao.module.system.api.loan.dto.LoanDTO;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;

import java.math.BigDecimal;
import java.util.List;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.smartfactory.enums.ErrorCodeConstants.STAFF_SALARY_EXISTS;
import static cn.iocoder.yudao.module.smartfactory.enums.ErrorCodeConstants.STAFF_SALARY_NOT_EXISTS;

/**
 * 厂区员工工资 Service 实现类
 *
 * @author 符溶馨
 */
@Service
@Validated
public class StaffSalaryServiceImpl implements StaffSalaryService {

    @Resource
    private StaffSalaryMapper staffSalaryMapper;

    @Resource
    private StaffMapper staffMapper;

    @Resource
    private LoanApi loanApi;

    @Override
    public Long createStaffSalary(StaffSalarySaveReqVO createReqVO) {

        // 校验当前月份工资是否存在
        Long count = staffSalaryMapper.selectCount(new LambdaQueryWrapperX<StaffSalaryDO>()
                .eq(StaffSalaryDO::getStaffId, createReqVO.getStaffId())
                .eq(StaffSalaryDO::getMonth, createReqVO.getMonth()));
        if (count > 0L) {
            throw exception(STAFF_SALARY_EXISTS);
        }

        // 插入
        StaffSalaryDO staffSalaryDO = BeanUtils.toBean(createReqVO, StaffSalaryDO.class);
        staffSalaryMapper.insert(staffSalaryDO);

        return staffSalaryDO.getId();
    }

    @Override
    public void updateStaffSalary(StaffSalarySaveReqVO updateReqVO) {
        // 校验存在
        validateStaffSalaryExists(updateReqVO.getId());
        // 更新
        StaffSalaryDO updateObj = BeanUtils.toBean(updateReqVO, StaffSalaryDO.class);
        staffSalaryMapper.updateById(updateObj);
    }

    @Override
    public void deleteStaffSalary(Long id) {
        // 校验存在
        validateStaffSalaryExists(id);
        // 删除
        staffSalaryMapper.deleteById(id);
    }

    private StaffSalaryDO validateStaffSalaryExists(Long id) {
        StaffSalaryDO staffSalaryDO = staffSalaryMapper.selectById(id);
        if (staffSalaryDO == null) {
            throw exception(STAFF_SALARY_NOT_EXISTS);
        }
        return staffSalaryDO;
    }

    @Override
    public StaffSalaryDO getStaffSalary(Long id) {
        return staffSalaryMapper.selectById(id);
    }

    @Override
    public PageResult<StaffSalaryRespVO> getStaffSalaryPage(StaffSalaryPageReqVO pageReqVO) {

        if (pageReqVO.getMonth() != null) {
            return staffMapper.selectSalaryPage(pageReqVO);
        }else {
            return staffSalaryMapper.selectSalaryPage(pageReqVO);
        }
    }

    @Override
    public void updateStatus(StaffSalarySaveReqVO updateReqVO) {

        // 校验存在
        StaffSalaryDO staffSalaryDO = validateStaffSalaryExists(updateReqVO.getId());
        // 更新
        StaffSalaryDO updateObj = BeanUtils.toBean(updateReqVO, StaffSalaryDO.class);
        staffSalaryMapper.updateById(updateObj);

        // 判断是否有归还借支金额
        if (staffSalaryDO.getReturnAmount().compareTo(BigDecimal.ZERO) > 0) {
            // 更新借支管理中 人员的归还金额
            LoanDTO updateDO = new LoanDTO()
                    .setUserId(staffSalaryDO.getStaffId())
                    .setAmount(BigDecimal.ZERO)
                    .setReturnAmount(staffSalaryDO.getReturnAmount());
            loanApi.createLoan(updateDO);
        }
    }

    @Override
    public List<StaffSalaryDO> getListByStaffId(Long staffId) {

        return staffSalaryMapper.selectList(new LambdaQueryWrapperX<StaffSalaryDO>()
                .eq(StaffSalaryDO::getStaffId, staffId)
                .eq(StaffSalaryDO::getStatus, 1));
    }
}